import { Tab, TabList, TabPanel, TabPanels, Tabs } from "@chakra-ui/react";
import { models } from "@/constant/models.ts";
import useModelStore from "@/store/useModelStore.ts";
import TextureSelectorPanel from "./TextureSelectorPanel";
import LogoPanel from "./LogoPanel";
import TextLabelPanel from "./TextLabelPanel";
import { useState } from "react";

const RightPanel = () => {
  const activeModel = useModelStore((state) => state.activeModel);
  const setActiveModel = useModelStore((state) => state.setActiveModel);

  const [activeTab, setActiveTab] = useState(0);
  return (
    <>
      <Tabs variant={"enclosed"} index={activeTab}>
        <TabList>
          <Tab
            className={`${
              activeTab === 0 ? "!rounded-none bg-white shadow-lg" : ""
            }`}
            onClick={() => {
              setActiveTab(() => 0);
            }}
          >
            版型
          </Tab>
          <Tab
            className={`${
              activeTab === 1 ? "!rounded-none bg-white shadow-lg" : ""
            }`}
            onClick={() => {
              setActiveTab(() => 1);
            }}
          >
            纹理
          </Tab>
          <Tab
            className={`${
              activeTab === 2 ? "!rounded-none bg-white shadow-lg" : ""
            }`}
            onClick={() => {
              setActiveTab(() => 2);
            }}
          >
            文字
          </Tab>
          <Tab
            className={`${
              activeTab === 3 ? "!rounded-none bg-white shadow-lg" : ""
            }`}
            onClick={() => {
              setActiveTab(() => 3);
            }}
          >
            LOGO
          </Tab>
        </TabList>
        <TabPanels>
          <TabPanel>
            <div className={"grid w-full grid-cols-6 gap-4"}>
              {models.map((model, index) => (
                <div
                  key={model.path}
                  onClick={() => {
                    setActiveModel(index);
                  }}
                  className={`${
                    activeModel === index ? "border-2 border-green-300" : ""
                  } flex aspect-square items-center justify-center overflow-hidden rounded-xl bg-blue-100`}
                >
                  <img
                    className="h-full w-full object-cover"
                    src={models[index].icon}
                    alt={""}
                  />
                </div>
              ))}
            </div>
          </TabPanel>
          <TabPanel>
            <TextureSelectorPanel />
          </TabPanel>
          ;
          <TabPanel>
            <TextLabelPanel />
          </TabPanel>
          ;
          <TabPanel>
            <LogoPanel />
          </TabPanel>
          ;
        </TabPanels>
      </Tabs>
    </>
  );
};

export default RightPanel;
