import { Canvas } from "@react-three/fiber";
import { TextureLoader, Vector3 } from "three";
import { Environment, OrbitControls } from "@react-three/drei";
import useModelStore from "@/store/useModelStore.ts";
import { models } from "@/constant/models.ts";
import { useGLTF } from "@react-three/drei/core/useGLTF";
import { textures } from "@/constant/textures.ts";
import Sticker from "./Sticker";
import { useState, useEffect, useRef } from "react";

const CapModel = () => {
  const activeModel = useModelStore((state) => state.activeModel);
  const activeTextures = useModelStore((state) => state.activeTextures);
  const decalDragging = useModelStore((state) => state.decalDragging);
  const activeDecal = useModelStore((state) => state.activeDecal);
  const setDecalPosition = useModelStore((state) => state.setDecalPosition);
  const [isAltDown, setIsAltDown] = useState(false);
  const ctrlRef = useRef(null);
  useEffect(() => {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-expect-error
    ctrlRef.current?.reset();
  }, [activeModel]);

  useEffect(() => {
    function handleKeyDown(event: KeyboardEvent) {
      if (event.key === "Alt") {
        setIsAltDown(true);
      }
    }

    function handleKeyUp(event: KeyboardEvent) {
      if (event.key === "Alt") {
        setIsAltDown(false);
      }
    }

    // 监听键盘按下和释放事件
    window.addEventListener("keydown", handleKeyDown);
    window.addEventListener("keyup", handleKeyUp);

    // 组件卸载时移除事件监听器
    return () => {
      window.removeEventListener("keydown", handleKeyDown);
      window.removeEventListener("keyup", handleKeyUp);
    };
  }, []);

  const { nodes } = useGLTF(models[activeModel].path);
  // const [ry, setRy] = useState(0);
  return (
    <>
      <group
        scale={models[activeModel].scale}
        onClick={(ev) => {
          console.log(ev.face?.normal);
          const angle = ev.face?.normal.angleTo(new Vector3(0, 1, 0));
          // if (angle && angle < Math.PI) {
          //   angle = 2 * Math.PI - angle;
          // }
          console.log(angle ?? 0 / Math.PI);
        }}
        onPointerMove={(ev) => {
          ev.stopPropagation();
          if (!decalDragging) return;
          if (activeDecal) {
            const { x, y, z } = ev.point;
            const pos = new Vector3(
              x / models[activeModel].scale,
              y / models[activeModel].scale,
              z / models[activeModel].scale,
            );
            setDecalPosition(activeDecal, pos);
          }
        }}
      >
        {Object.keys(nodes).map((keyName) => {
          if (nodes[keyName].type === "Mesh") {
            const areaIndex = models[activeModel].mesh.findIndex(
              (el) => el.name === nodes[keyName].name,
            );
            return (
              <mesh
                key={keyName}
                onDoubleClick={(ev) => {
                  ev.stopPropagation();

                  if (activeDecal) {
                    const { x, y, z } = ev.point;
                    const pos = new Vector3(
                      x / models[activeModel].scale,
                      y / models[activeModel].scale,
                      z / models[activeModel].scale,
                    );
                    // console.log(ev.point.transformDirection(ev.intersections[0].object.matrixWorld));
                    const normal = ev.face?.normal;
                    // normal?.transformDirection(ev.intersections[0].object.matrixWorld)
                    // normal?.applyQuaternion(ev.object.quaternion)
                    // normal?.normalize();
                    console.log(normal);
                    // const _ry = y + 0.5;
                    // setRy(_ry);
                    // console.log(normal?.angleTo(new Vector3(0, 1, 0)));

                    // const y_a = normal?.angleTo(new Vector3(0, 1, 0))
                    // const x_a = normal?.angleTo(new Vector3(1, 0, 0))
                    // const z_a = normal?.angleTo(new Vector3(0, 0, 1))
                    // console.log(x_a, y_a, z_a);
                    setDecalPosition(activeDecal, pos);
                  }
                }}
                castShadow
                receiveShadow
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-expect-error
                geometry={nodes[keyName].geometry}
                dispose={null}
              >
                {areaIndex > -1 ? (
                  <>
                    <meshStandardMaterial
                      map={new TextureLoader().load(
                        textures[activeTextures[areaIndex]].path,
                      )}
                    />
                    <Stickers />
                  </>
                ) : (
                  <meshStandardMaterial color="gray" />
                )}
              </mesh>
            );
          }
        })}
      </group>
      <OrbitControls
        ref={ctrlRef}
        makeDefault={true}
        enabled={!decalDragging}
        enableZoom={!isAltDown}
      />
      ;
    </>
  );
};

export const Stickers = () => {
  const decals = useModelStore((state) => state.decals);

  return (
    <>
      {decals.map((decal) => (
        <Sticker decal={decal} key={decal.id} />
      ))}
    </>
  );
};

const ThreeScene = () => {
  return (
    <Canvas>
      <ambientLight></ambientLight>
      <pointLight position={[10, 10, 10]}></pointLight>
      <perspectiveCamera />
      <CapModel />
      <axesHelper />
      <Environment files={"/venice_sunset_1k.hdr"} background={false} />
    </Canvas>
  );
};

export default ThreeScene;
